@@ -1,8 +1,8 @@
 /* aci.c - routines to parse and check acl's */
-/* $OpenLDAP: pkg/ldap/servers/slapd/aci.c,v 1.14.2.4 2007/08/31 23:13:57 quanah Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2007 The OpenLDAP Foundation.
+ * Copyright 1998-2021 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -281,20 +281,19 @@ aci_list_get_attr_rights(
 			Debug( LDAP_DEBUG_ACL,
 				"        <= aci_list_get_attr_rights "
 				"test %s for %s -> failed\n",
-				bv.bv_val, attr->bv_val, 0 );
+				bv.bv_val, attr->bv_val );
 			continue;
 		}
 
 		Debug( LDAP_DEBUG_ACL,
 			"        <= aci_list_get_attr_rights "
 			"test %s for %s -> ok\n",
-			bv.bv_val, attr->bv_val, 0 );
+			bv.bv_val, attr->bv_val );
 
 		if ( acl_get_part( list, i, ';', &bv ) < 0 ) {
 			Debug( LDAP_DEBUG_ACL,
 				"        <= aci_list_get_attr_rights "
-				"test no rights\n",
-				0, 0, 0 );
+				"test no rights\n" );
 			continue;
 		}
 
@@ -302,7 +301,7 @@ aci_list_get_attr_rights(
 		Debug( LDAP_DEBUG_ACL,
 			"        <= aci_list_get_attr_rights "
 			"rights %s to mask 0x%x\n",
-			bv.bv_val, mask, 0 );
+			bv.bv_val, mask );
 	}
 
 	return mask;
@@ -345,9 +344,12 @@ aci_list_get_rights(
 			continue;
 		}
 
-		found = 1;
 		*mask |= aci_list_get_attr_rights( &perm, attr, val );
 		*mask |= aci_list_get_attr_rights( &perm, &aci_bv[ ACI_BV_BR_ALL ], NULL );
+
+		if ( *mask != ACL_PRIV_NONE ) { 
+			found = 1;
+		}
 	}
 
 	return found;
@@ -397,11 +399,15 @@ aci_group_member (
 	if ( grp_oc != NULL && grp_ad != NULL ) {
 		char		buf[ ACI_BUF_SIZE ];
 		struct berval	bv, ndn;
+		AclRegexMatches amatches = { 0 };
+
+		amatches.dn_count = nmatch;
+		AC_MEMCPY( amatches.dn_data, matches, sizeof( amatches.dn_data ) );
 
 		bv.bv_len = sizeof( buf ) - 1;
 		bv.bv_val = (char *)&buf;
 		if ( acl_string_expand( &bv, &subjdn,
-				e->e_ndn, nmatch, matches ) )
+				&e->e_nname, NULL, &amatches ) )
 		{
 			rc = LDAP_OTHER;
 			goto done;
@@ -439,7 +445,9 @@ aci_mask(
 				opts,
 				sdn;
 	int			rc;
-		
+
+	ACL_INIT( *grant );
+	ACL_INIT( *deny );
 
 	assert( !BER_BVISNULL( &desc->ad_cname ) );
 
@@ -592,11 +600,10 @@ aci_mask(
 				at != NULL;
 				at = attrs_find( at->a_next, ad ) )
 		{
-			if ( value_find_ex( ad,
+			if ( attr_valfind( at, 
 				SLAP_MR_ATTRIBUTE_VALUE_NORMALIZED_MATCH |
 					SLAP_MR_ASSERTED_VALUE_NORMALIZED_MATCH,
-				at->a_nvals,
-				&op->o_ndn, op->o_tmpmemctx ) == 0 )
+				&op->o_ndn, NULL, op->o_tmpmemctx ) == 0 )
 			{
 				rc = 1;
 				break;
@@ -722,7 +729,7 @@ aci_init( void )
 	rc = register_at( aci_at.desc, aci_at.ad, 0 );
 	if ( rc != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_ANY,
-			"aci_init: at_register failed\n", 0, 0, 0 );
+			"aci_init: at_register failed\n" );
 		return rc;
 	}
 
@@ -842,7 +849,7 @@ dynacl_aci_mask(
 		
 		Debug( LDAP_DEBUG_ACL, "        <= aci_mask grant %s deny %s\n",
 			  accessmask2str( tgrant, accessmaskbuf, 1 ), 
-			  accessmask2str( tdeny, accessmaskbuf1, 1 ), 0 );
+			  accessmask2str( tdeny, accessmaskbuf1, 1 ) );
 	}
 
 	/* If the entry level aci didn't contain anything valid for the 
@@ -880,7 +887,7 @@ dynacl_aci_mask(
 				op->o_ndn = op->o_bd->be_rootndn;
 			}
 
-			Debug( LDAP_DEBUG_ACL, "        checking ACI of \"%s\"\n", parent_ndn.bv_val, 0, 0 );
+			Debug( LDAP_DEBUG_ACL, "        checking ACI of \"%s\"\n", parent_ndn.bv_val );
 			ret = backend_attribute( op, NULL, &parent_ndn, ad, &bvals, ACL_AUTH );
 
 			if ( !BER_BVISNULL( &op->o_bd->be_rootndn ) ) {
@@ -913,7 +920,7 @@ dynacl_aci_mask(
 					}
 					Debug( LDAP_DEBUG_ACL, "<= aci_mask grant %s deny %s\n", 
 						accessmask2str( tgrant, accessmaskbuf, 1 ),
-						accessmask2str( tdeny, accessmaskbuf1, 1 ), 0 );
+						accessmask2str( tdeny, accessmaskbuf1, 1 ) );
 				}
 				break;
 
@@ -921,13 +928,13 @@ dynacl_aci_mask(
 				/* just go on if the aci-Attribute is not present in
 				 * the current entry 
 				 */
-				Debug( LDAP_DEBUG_ACL, "no such attribute\n", 0, 0, 0 );
+				Debug( LDAP_DEBUG_ACL, "no such attribute\n" );
 				stop = 0;
 				break;
 
 			case LDAP_NO_SUCH_OBJECT:
 				/* We have reached the base object */
-				Debug( LDAP_DEBUG_ACL, "no such object\n", 0, 0, 0 );
+				Debug( LDAP_DEBUG_ACL, "no such object\n" );
 				stop = 1;
 				break;
 
@@ -1039,7 +1046,7 @@ static int
 OpenLDAPaciValidatePerms(
 	struct berval *perms ) 
 {
-	int		i;
+	ber_len_t	i;
 
 	for ( i = 0; i < perms->bv_len; ) {
 		switch ( perms->bv_val[ i ] ) {
@@ -1052,7 +1059,7 @@ OpenLDAPaciValidatePerms(
 			break;
 
 		default:
-		        Debug( LDAP_DEBUG_ACL, "aciValidatePerms: perms needs to be one of x,d,c,s,r,w in '%s'\n", perms->bv_val, 0, 0 );
+		        Debug( LDAP_DEBUG_ACL, "aciValidatePerms: perms needs to be one of x,d,c,s,r,w in '%s'\n", perms->bv_val );
 			return LDAP_INVALID_SYNTAX;
 		}
 
@@ -1066,7 +1073,7 @@ OpenLDAPaciValidatePerms(
 		assert( i != perms->bv_len );
 
 		if ( perms->bv_val[ i ] != ',' ) {
-		        Debug( LDAP_DEBUG_ACL, "aciValidatePerms: missing comma in '%s'\n", perms->bv_val, 0, 0 );
+		        Debug( LDAP_DEBUG_ACL, "aciValidatePerms: missing comma in '%s'\n", perms->bv_val );
 			return LDAP_INVALID_SYNTAX;
 		}
 
@@ -1095,7 +1102,7 @@ OpenLDAPaciValidateRight(
 	if ( acl_get_part( action, 0, ';', &bv ) < 0 ||
 		bv_getcaseidx( &bv, ACIgrantdeny ) == -1 )
 	{
-		Debug( LDAP_DEBUG_ACL, "aciValidateRight: '%s' must be either 'grant' or 'deny'\n", bv.bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidateRight: '%s' must be either 'grant' or 'deny'\n", bv.bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1129,13 +1136,13 @@ OpenLDAPaciValidateRight(
 				{
 					if ( slap_bv2ad( &attr, &ad, &text ) != LDAP_SUCCESS ) 
 					{
-						Debug( LDAP_DEBUG_ACL, "aciValidateRight: unknown attribute: '%s'\n", attr.bv_val, 0, 0 );
+						Debug( LDAP_DEBUG_ACL, "aciValidateRight: unknown attribute: '%s'\n", attr.bv_val );
 						return LDAP_INVALID_SYNTAX;
 					}
 				} else {
 					if ( slap_bv2ad( &left, &ad, &text ) != LDAP_SUCCESS ) 
 					{
-						Debug( LDAP_DEBUG_ACL, "aciValidateRight: unknown attribute: '%s'\n", left.bv_val, 0, 0 );
+						Debug( LDAP_DEBUG_ACL, "aciValidateRight: unknown attribute: '%s'\n", left.bv_val );
 						return LDAP_INVALID_SYNTAX;
 					}
 				}
@@ -1148,7 +1155,7 @@ OpenLDAPaciValidateRight(
 		return LDAP_SUCCESS;
 
 	} else {
-		Debug( LDAP_DEBUG_ACL, "aciValidateRight: perms:attr need to be pairs in '%s'\n", action->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidateRight: perms:attr need to be pairs in '%s'\n", action->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1169,12 +1176,12 @@ OpenLDAPaciNormalizeRight(
 
 	/* grant|deny */
 	if ( acl_get_part( action, 0, ';', &grantdeny ) < 0 ) {
-	        Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: missing ';' in '%s'\n", action->bv_val, 0, 0 );
+	        Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: missing ';' in '%s'\n", action->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 	idx = bv_getcaseidx( &grantdeny, ACIgrantdeny );
 	if ( idx == -1 ) {
-	        Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: '%s' must be grant or deny\n", grantdeny.bv_val, 0, 0 );
+	        Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: '%s' must be grant or deny\n", grantdeny.bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1212,17 +1219,17 @@ OpenLDAPaciNormalizeRight(
 				{
 					ad = NULL;
 					text = NULL;
-					/* openldap 2.1 aci compabitibility [entry] -> entry */
+					/* openldap 2.1 aci compatibility [entry] -> entry */
 					if ( ber_bvstrcasecmp( &attr, &aci_bv[ ACI_BV_BR_ENTRY ] ) == 0 ) {
 						ad = &adstatic;
 						adstatic.ad_cname = aci_bv[ ACI_BV_ENTRY ];
 
-					/* openldap 2.1 aci compabitibility [children] -> children */
+					/* openldap 2.1 aci compatibility [children] -> children */
 					} else if ( ber_bvstrcasecmp( &attr, &aci_bv[ ACI_BV_BR_CHILDREN ] ) == 0 ) {
 						ad = &adstatic;
 						adstatic.ad_cname = aci_bv[ ACI_BV_CHILDREN ];
 
-					/* openldap 2.1 aci compabitibility [all] -> only [all] */
+					/* openldap 2.1 aci compatibility [all] -> only [all] */
 					} else if ( ber_bvstrcasecmp( &attr, &aci_bv[ ACI_BV_BR_ALL ] ) == 0 ) {
 						ber_memfree_x( nattrs.bv_val, ctx );
 						nattrs = aci_bv[ ACI_BV_BR_ALL ];
@@ -1235,7 +1242,7 @@ OpenLDAPaciNormalizeRight(
 						if ( slap_bv2ad( &attr, &ad, &text ) != LDAP_SUCCESS ) 
 						{
 							ber_memfree_x( nattrs.bv_val, ctx );
-							Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: unknown attribute: '%s'\n", attr.bv_val, 0, 0 );
+							Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: unknown attribute: '%s'\n", attr.bv_val );
 							return LDAP_INVALID_SYNTAX;
 						}
 
@@ -1243,7 +1250,7 @@ OpenLDAPaciNormalizeRight(
 						if ( slap_bv2ad( &left, &ad, &text ) != LDAP_SUCCESS ) 
 						{
 							ber_memfree_x( nattrs.bv_val, ctx );
-							Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: unknown attribute: '%s'\n", left.bv_val, 0, 0 );
+							Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: unknown attribute: '%s'\n", left.bv_val );
 							return LDAP_INVALID_SYNTAX;
 						}
 					}
@@ -1251,7 +1258,7 @@ OpenLDAPaciNormalizeRight(
 				
 					len = nattrs.bv_len + ( !BER_BVISEMPTY( &nattrs ) ? STRLENOF( "," ) : 0 )
 				      		+ ad->ad_cname.bv_len;
-					nattrs.bv_val = ber_memrealloc_x( nattrs.bv_val, len + 1, ctx );
+					nattrs.bv_val = slap_sl_realloc( nattrs.bv_val, len + 1, ctx );
 	                        	ptr = &nattrs.bv_val[ nattrs.bv_len ];
 					if ( !BER_BVISEMPTY( &nattrs ) ) {
 						*ptr++ = ',';
@@ -1263,7 +1270,7 @@ OpenLDAPaciNormalizeRight(
 
 			}
 
-			naction->bv_val = ber_memrealloc_x( naction->bv_val,
+			naction->bv_val = slap_sl_realloc( naction->bv_val,
 				naction->bv_len + STRLENOF( ";" )
 				+ perms.bv_len + STRLENOF( ";" )
 				+ nattrs.bv_len + 1,
@@ -1290,7 +1297,7 @@ OpenLDAPaciNormalizeRight(
 		return LDAP_SUCCESS;
 
 	} else {
-		Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: perms:attr need to be pairs in '%s'\n", action->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciNormalizeRight: perms:attr need to be pairs in '%s'\n", action->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 }
@@ -1338,7 +1345,7 @@ OpenLDAPaciNormalizeRights(
 			*nactions = nbv;
 
 		} else {
-			nactions->bv_val = ber_memrealloc_x( nactions->bv_val,
+			nactions->bv_val = slap_sl_realloc( nactions->bv_val,
 				nactions->bv_len + STRLENOF( "$" )
 				+ nbv.bv_len + 1,
 				ctx );
@@ -1403,7 +1410,7 @@ OpenLDAPaciValidate(
 	int		rc;
 	
 	if ( BER_BVISEMPTY( val ) ) {
-		Debug( LDAP_DEBUG_ACL, "aciValidatet: value is empty\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidatet: value is empty\n" );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1415,7 +1422,7 @@ OpenLDAPaciValidate(
 		 * I'd replace it with X-ORDERED VALUES so that
 		 * it's guaranteed values are maintained and used
 		 * in the desired order */
-		Debug( LDAP_DEBUG_ACL, "aciValidate: invalid oid '%s'\n", oid.bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidate: invalid oid '%s'\n", oid.bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1423,7 +1430,7 @@ OpenLDAPaciValidate(
 	if ( acl_get_part( val, 1, '#', &scope ) < 0 || 
 		bv_getcaseidx( &scope, OpenLDAPaciscopes ) == -1 )
 	{
-		Debug( LDAP_DEBUG_ACL, "aciValidate: invalid scope '%s'\n", scope.bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidate: invalid scope '%s'\n", scope.bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1436,7 +1443,7 @@ OpenLDAPaciValidate(
 
 	/* type */
 	if ( acl_get_part( val, 3, '#', &type ) < 0 ) {
-		Debug( LDAP_DEBUG_ACL, "aciValidate: missing type in '%s'\n", val->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidate: missing type in '%s'\n", val->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 	idx = bv_getcaseidx( &type, OpenLDAPacitypes );
@@ -1444,13 +1451,13 @@ OpenLDAPaciValidate(
 		struct berval	isgr;
 
 		if ( acl_get_part( &type, 0, '/', &isgr ) < 0 ) {
-			Debug( LDAP_DEBUG_ACL, "aciValidate: invalid type '%s'\n", type.bv_val, 0, 0 );
+			Debug( LDAP_DEBUG_ACL, "aciValidate: invalid type '%s'\n", type.bv_val );
 			return LDAP_INVALID_SYNTAX;
 		}
 
 		idx = bv_getcaseidx( &isgr, OpenLDAPacitypes );
 		if ( idx == -1 || idx >= LAST_OPTIONAL ) {
-			Debug( LDAP_DEBUG_ACL, "aciValidate: invalid type '%s'\n", isgr.bv_val, 0, 0 );
+			Debug( LDAP_DEBUG_ACL, "aciValidate: invalid type '%s'\n", isgr.bv_val );
 			return LDAP_INVALID_SYNTAX;
 		}
 	}
@@ -1458,7 +1465,7 @@ OpenLDAPaciValidate(
 	/* subject */
 	bv_get_tail( val, &type, &subject );
 	if ( subject.bv_val[ 0 ] != '#' ) {
-		Debug( LDAP_DEBUG_ACL, "aciValidate: missing subject in '%s'\n", val->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciValidate: missing subject in '%s'\n", val->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1469,13 +1476,13 @@ OpenLDAPaciValidate(
 
 			rc = slap_bv2ad( &subject, &ad, &text );
 			if ( rc != LDAP_SUCCESS ) {
-				Debug( LDAP_DEBUG_ACL, "aciValidate: unknown dn attribute '%s'\n", subject.bv_val, 0, 0 );
+				Debug( LDAP_DEBUG_ACL, "aciValidate: unknown dn attribute '%s'\n", subject.bv_val );
 				return LDAP_INVALID_SYNTAX;
 			}
 
 			if ( ad->ad_type->sat_syntax != slap_schema.si_syn_distinguishedName ) {
 				/* FIXME: allow nameAndOptionalUID? */
-				Debug( LDAP_DEBUG_ACL, "aciValidate: wrong syntax for dn attribute '%s'\n", subject.bv_val, 0, 0 );
+				Debug( LDAP_DEBUG_ACL, "aciValidate: wrong syntax for dn attribute '%s'\n", subject.bv_val );
 				return LDAP_INVALID_SYNTAX;
 			}
 		}
@@ -1509,13 +1516,13 @@ OpenLDAPaciValidate(
 
 				rc = slap_bv2ad( &atbv, &ad, &text );
 				if ( rc != LDAP_SUCCESS ) {
-				        Debug( LDAP_DEBUG_ACL, "aciValidate: unknown group attribute '%s'\n", atbv.bv_val, 0, 0 );
+				        Debug( LDAP_DEBUG_ACL, "aciValidate: unknown group attribute '%s'\n", atbv.bv_val );
 					return LDAP_INVALID_SYNTAX;
 				}
 			}
 
 			if ( oc_bvfind( &ocbv ) == NULL ) {
-			        Debug( LDAP_DEBUG_ACL, "aciValidate: unknown group '%s'\n", ocbv.bv_val, 0, 0 );
+			        Debug( LDAP_DEBUG_ACL, "aciValidate: unknown group '%s'\n", ocbv.bv_val );
 				return LDAP_INVALID_SYNTAX;
 			}
 		}
@@ -1523,7 +1530,7 @@ OpenLDAPaciValidate(
 
 	if ( BER_BVISEMPTY( &subject ) ) {
 		/* empty DN invalid */
-	        Debug( LDAP_DEBUG_ACL, "aciValidate: missing dn in '%s'\n", val->bv_val, 0, 0 );
+	        Debug( LDAP_DEBUG_ACL, "aciValidate: missing dn in '%s'\n", val->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1533,7 +1540,7 @@ OpenLDAPaciValidate(
 	/* FIXME: pass DN syntax? */
 	rc = dnValidate( NULL, &subject );
 	if ( rc != LDAP_SUCCESS ) {
-	        Debug( LDAP_DEBUG_ACL, "aciValidate: invalid dn '%s'\n", subject.bv_val, 0, 0 );
+	        Debug( LDAP_DEBUG_ACL, "aciValidate: invalid dn '%s'\n", subject.bv_val );
 	}
 	return rc;
 }
@@ -1562,7 +1569,7 @@ OpenLDAPaciPrettyNormal(
 	BER_BVZERO( out );
 
 	if ( BER_BVISEMPTY( val ) ) {
-		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: value is empty\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: value is empty\n" );
 		return LDAP_INVALID_SYNTAX;
 	}
 
@@ -1570,25 +1577,25 @@ OpenLDAPaciPrettyNormal(
 	if ( acl_get_part( val, 0, '#', &oid ) < 0 || 
 		numericoidValidate( NULL, &oid ) != LDAP_SUCCESS )
 	{
-		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid oid '%s'\n", oid.bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid oid '%s'\n", oid.bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 
 	/* scope: normalize by replacing with OpenLDAPaciscopes */
 	if ( acl_get_part( val, 1, '#', &scope ) < 0 ) {
-		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing scope in '%s'\n", val->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing scope in '%s'\n", val->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 	idx = bv_getcaseidx( &scope, OpenLDAPaciscopes );
 	if ( idx == -1 ) {
-		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid scope '%s'\n", scope.bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid scope '%s'\n", scope.bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 	scope = *OpenLDAPaciscopes[ idx ];
 
 	/* rights */
 	if ( acl_get_part( val, 2, '#', &rights ) < 0 ) {
-		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing rights in '%s'\n", val->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing rights in '%s'\n", val->bv_val );
 		return LDAP_INVALID_SYNTAX;
 	}
 	if ( OpenLDAPaciNormalizeRights( &rights, &nrights, ctx )
@@ -1599,7 +1606,7 @@ OpenLDAPaciPrettyNormal(
 
 	/* type */
 	if ( acl_get_part( val, 3, '#', &type ) < 0 ) {
-		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing type in '%s'\n", val->bv_val, 0, 0 );
+		Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing type in '%s'\n", val->bv_val );
 		rc = LDAP_INVALID_SYNTAX;
 		goto cleanup;
 	}
@@ -1608,14 +1615,14 @@ OpenLDAPaciPrettyNormal(
 		struct berval	isgr;
 
 		if ( acl_get_part( &type, 0, '/', &isgr ) < 0 ) {
-		        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid type '%s'\n", type.bv_val, 0, 0 );
+		        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid type '%s'\n", type.bv_val );
 			rc = LDAP_INVALID_SYNTAX;
 			goto cleanup;
 		}
 
 		idx = bv_getcaseidx( &isgr, OpenLDAPacitypes );
 		if ( idx == -1 || idx >= LAST_OPTIONAL ) {
-		        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid type '%s'\n", isgr.bv_val, 0, 0 );
+		        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid type '%s'\n", isgr.bv_val );
 			rc = LDAP_INVALID_SYNTAX;
 			goto cleanup;
 		}
@@ -1626,7 +1633,7 @@ OpenLDAPaciPrettyNormal(
 	bv_get_tail( val, &type, &subject );
 
 	if ( BER_BVISEMPTY( &subject ) || subject.bv_val[ 0 ] != '#' ) {
-	        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing subject in '%s'\n", val->bv_val, 0, 0 );
+	        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: missing subject in '%s'\n", val->bv_val );
 		rc = LDAP_INVALID_SYNTAX;
 		goto cleanup;
 	}
@@ -1647,7 +1654,7 @@ OpenLDAPaciPrettyNormal(
 			freesubject = 1;
 
 		} else {
-	                Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid subject dn '%s'\n", subject.bv_val, 0, 0 );
+	                Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid subject dn '%s'\n", subject.bv_val );
 			goto cleanup;
 		}
 
@@ -1680,7 +1687,7 @@ OpenLDAPaciPrettyNormal(
 	
 					rc = slap_bv2ad( &atbv, &ad, &text );
 					if ( rc != LDAP_SUCCESS ) {
-	                                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: unknown group attribute '%s'\n", atbv.bv_val, 0, 0 );
+	                                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: unknown group attribute '%s'\n", atbv.bv_val );
 						rc = LDAP_INVALID_SYNTAX;
 						goto cleanup;
 					}
@@ -1690,13 +1697,13 @@ OpenLDAPaciPrettyNormal(
 
 				oc = oc_bvfind( &ocbv );
 				if ( oc == NULL ) {
-                                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid group '%s'\n", ocbv.bv_val, 0, 0 );
+                                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: invalid group '%s'\n", ocbv.bv_val );
 					rc = LDAP_INVALID_SYNTAX;
 					goto cleanup;
 				}
 
 				bv.bv_len += STRLENOF( "/" ) + oc->soc_cname.bv_len;
-				bv.bv_val = ber_memalloc_x( bv.bv_len + 1, ctx );
+				bv.bv_val = slap_sl_malloc( bv.bv_len + 1, ctx );
 
 				ptr = bv.bv_val;
 				ptr = lutil_strncopy( ptr, ntype.bv_val, ntype.bv_len );
@@ -1726,19 +1733,25 @@ OpenLDAPaciPrettyNormal(
 
 		rc = slap_bv2ad( &subject, &ad, &text );
 		if ( rc != LDAP_SUCCESS ) {
-                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: unknown dn attribute '%s'\n", subject.bv_val, 0, 0 );
+                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: unknown dn attribute '%s'\n", subject.bv_val );
 			rc = LDAP_INVALID_SYNTAX;
 			goto cleanup;
 		}
 
 		if ( ad->ad_type->sat_syntax != slap_schema.si_syn_distinguishedName ) {
 			/* FIXME: allow nameAndOptionalUID? */
-                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: wrong syntax for dn attribute '%s'\n", subject.bv_val, 0, 0 );
+                        Debug( LDAP_DEBUG_ACL, "aciPrettyNormal: wrong syntax for dn attribute '%s'\n", subject.bv_val );
 			rc = LDAP_INVALID_SYNTAX;
 			goto cleanup;
 		}
 
 		nsubject = ad->ad_cname;
+
+	} else if ( OpenLDAPacitypes[ idx ] == &aci_bv[ ACI_BV_SET ]
+		|| OpenLDAPacitypes[ idx ] == &aci_bv[ ACI_BV_SET_REF ] )
+	{
+		/* NOTE: dunno how to normalize it... */
+		nsubject = subject;
 	}
 
 
@@ -1749,7 +1762,7 @@ OpenLDAPaciPrettyNormal(
 		+ ntype.bv_len + STRLENOF( "#" )
 		+ nsubject.bv_len;
 
-	out->bv_val = ber_memalloc_x( out->bv_len + 1, ctx );
+	out->bv_val = slap_sl_malloc( out->bv_len + 1, ctx );
 	ptr = lutil_strncopy( out->bv_val, oid.bv_val, oid.bv_len );
 	ptr[ 0 ] = '#';
 	ptr++;
