@@ -1,8 +1,8 @@
 /* init.c - initialize various things */
-/* $OpenLDAP: pkg/ldap/servers/slapd/init.c,v 1.97.2.4 2007/08/31 23:13:59 quanah Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2007 The OpenLDAP Foundation.
+ * Copyright 1998-2021 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,19 +55,15 @@ int		ldap_syslog_level = LOG_DEBUG;
 
 BerVarray default_referral = NULL;
 
-struct berval AllUser = BER_BVC( LDAP_ALL_USER_ATTRIBUTES );
-struct berval AllOper = BER_BVC( LDAP_ALL_OPERATIONAL_ATTRIBUTES );
-struct berval NoAttrs = BER_BVC( LDAP_NO_ATTRS );
-
 /*
  * global variables that need mutex protection
  */
 ldap_pvt_thread_pool_t	connection_pool;
-int			connection_pool_max = SLAP_MAX_WORKER_THREADS;
+int		connection_pool_max = SLAP_MAX_WORKER_THREADS;
+int		connection_pool_queues = 1;
 int		slap_tool_thread_max = 1;
-ldap_pvt_thread_mutex_t	gmtime_mutex;
 
-slap_counters_t			slap_counters;
+slap_counters_t			slap_counters, *slap_counters_list;
 
 static const char* slap_name = NULL;
 int slapMode = SLAP_UNDEFINED_MODE;
@@ -76,7 +72,6 @@ int
 slap_init( int mode, const char *name )
 {
 	int rc;
-	int i;
 
 	assert( mode );
 
@@ -99,7 +94,7 @@ slap_init( int mode, const char *name )
 		slap_debug |= LDAP_DEBUG_NONE;
 		Debug( LDAP_DEBUG_ANY,
 		    "%s: module_init failed\n",
-			name, 0, 0 );
+			name );
 		return 1;
 	}
 #endif
@@ -108,7 +103,15 @@ slap_init( int mode, const char *name )
 		slap_debug |= LDAP_DEBUG_NONE;
 		Debug( LDAP_DEBUG_ANY,
 		    "%s: slap_schema_init failed\n",
-		    name, 0, 0 );
+		    name );
+		return 1;
+	}
+
+	if ( filter_init() != 0 ) {
+		slap_debug |= LDAP_DEBUG_NONE;
+		Debug( LDAP_DEBUG_ANY,
+		    "%s: filter_init failed\n",
+		    name );
 		return 1;
 	}
 
@@ -116,7 +119,7 @@ slap_init( int mode, const char *name )
 		slap_debug |= LDAP_DEBUG_NONE;
 		Debug( LDAP_DEBUG_ANY,
 		    "%s: entry_init failed\n",
-		    name, 0, 0 );
+		    name );
 		return 1;
 	}
 
@@ -128,36 +131,19 @@ slap_init( int mode, const char *name )
 	case SLAP_TOOL_MODE:
 		Debug( LDAP_DEBUG_TRACE,
 			"%s init: initiated %s.\n",	name,
-			(mode & SLAP_MODE) == SLAP_TOOL_MODE ? "tool" : "server",
-			0 );
+			(mode & SLAP_MODE) == SLAP_TOOL_MODE ? "tool" : "server" );
 
 		slap_name = name;
 
-		ldap_pvt_thread_pool_init( &connection_pool,
-				connection_pool_max, 0);
+		ldap_pvt_thread_pool_init_q( &connection_pool,
+				connection_pool_max, 0, connection_pool_queues);
 
-		ldap_pvt_thread_mutex_init( &slap_counters.sc_sent_mutex );
-		ldap_pvt_thread_mutex_init( &slap_counters.sc_ops_mutex );
-		ldap_pvt_mp_init( slap_counters.sc_bytes );
-		ldap_pvt_mp_init( slap_counters.sc_pdu );
-		ldap_pvt_mp_init( slap_counters.sc_entries );
-		ldap_pvt_mp_init( slap_counters.sc_refs );
-
-		ldap_pvt_mp_init( slap_counters.sc_ops_initiated );
-		ldap_pvt_mp_init( slap_counters.sc_ops_completed );
+		slap_counters_init( &slap_counters );
 
 		ldap_pvt_thread_mutex_init( &slapd_rq.rq_mutex );
 		LDAP_STAILQ_INIT( &slapd_rq.task_list );
 		LDAP_STAILQ_INIT( &slapd_rq.run_list );
 
-#ifdef SLAPD_MONITOR
-		for ( i = 0; i < SLAP_OP_LAST; i++ ) {
-			ldap_pvt_mp_init( slap_counters.sc_ops_initiated_[ i ] );
-			ldap_pvt_mp_init( slap_counters.sc_ops_completed_[ i ] );
-		}
-#endif /* SLAPD_MONITOR */
-
-		ldap_pvt_thread_mutex_init( &gmtime_mutex );
 		slap_passwd_init();
 
 		rc = slap_sasl_init();
@@ -173,7 +159,7 @@ slap_init( int mode, const char *name )
 	default:
 		slap_debug |= LDAP_DEBUG_NONE;
 		Debug( LDAP_DEBUG_ANY,
-			"%s init: undefined mode (%d).\n", name, mode, 0 );
+			"%s init: undefined mode (%d).\n", name, mode );
 
 		rc = 1;
 		break;
@@ -183,25 +169,15 @@ slap_init( int mode, const char *name )
 		slap_debug |= LDAP_DEBUG_NONE;
 		Debug( LDAP_DEBUG_ANY,
 		    "%s: slap_controls_init failed\n",
-		    name, 0, 0 );
+		    name );
 		return 1;
 	}
 
-#ifdef HAVE_TLS
-	/* Library defaults to full certificate checking. This is correct when
-	 * a client is verifying a server because all servers should have a
-	 * valid cert. But few clients have valid certs, so we want our default
-	 * to be no checking. The config file can override this as usual.
-	 */
-	rc = 0;
-	(void) ldap_pvt_tls_set_option( NULL, LDAP_OPT_X_TLS_REQUIRE_CERT, &rc );
-#endif
-
 	if ( frontend_init() ) {
 		slap_debug |= LDAP_DEBUG_NONE;
 		Debug( LDAP_DEBUG_ANY,
 		    "%s: frontend_init failed\n",
-		    name, 0, 0 );
+		    name );
 		return 1;
 	}
 
@@ -209,7 +185,7 @@ slap_init( int mode, const char *name )
 		slap_debug |= LDAP_DEBUG_NONE;
 		Debug( LDAP_DEBUG_ANY,
 		    "%s: overlay_init failed\n",
-		    name, 0, 0 );
+		    name );
 		return 1;
 	}
 
@@ -217,7 +193,7 @@ slap_init( int mode, const char *name )
 		slap_debug |= LDAP_DEBUG_NONE;
 		Debug( LDAP_DEBUG_ANY,
 		    "%s: glue/subordinate init failed\n",
-		    name, 0, 0 );
+		    name );
 
 		return 1;
 	}
@@ -226,7 +202,7 @@ slap_init( int mode, const char *name )
 		slap_debug |= LDAP_DEBUG_NONE;
 		Debug( LDAP_DEBUG_ANY,
 		    "%s: acl_init failed\n",
-		    name, 0, 0 );
+		    name );
 		return 1;
 	}
 
@@ -235,19 +211,25 @@ slap_init( int mode, const char *name )
 
 int slap_startup( Backend *be )
 {
+	int rc;
 	Debug( LDAP_DEBUG_TRACE,
 		"%s startup: initiated.\n",
-		slap_name, 0, 0 );
+		slap_name );
 
-
-	return backend_startup( be );
+	rc = backend_startup( be );
+	if ( !rc && ( slapMode & SLAP_SERVER_MODE ))
+		slapMode |= SLAP_SERVER_RUNNING;
+	return rc;
 }
 
 int slap_shutdown( Backend *be )
 {
 	Debug( LDAP_DEBUG_TRACE,
 		"%s shutdown: initiated\n",
-		slap_name, 0, 0 );
+		slap_name );
+
+	/* Make sure the pool stops now even if we did not start up fully */
+	ldap_pvt_thread_pool_close( &connection_pool, 1 );
 
 	/* let backends do whatever cleanup they need to do */
 	return backend_shutdown( be ); 
@@ -256,16 +238,17 @@ int slap_shutdown( Backend *be )
 int slap_destroy(void)
 {
 	int rc;
-	int i;
 
 	Debug( LDAP_DEBUG_TRACE,
 		"%s destroy: freeing system resources.\n",
-		slap_name, 0, 0 );
+		slap_name );
 
 	if ( default_referral ) {
 		ber_bvarray_free( default_referral );
 	}
 
+	ldap_pvt_thread_pool_free( &connection_pool );
+
 	/* clear out any thread-keys for the main thread */
 	ldap_pvt_thread_pool_context_reset( ldap_pvt_thread_pool_context());
 
@@ -280,29 +263,13 @@ int slap_destroy(void)
 
 	switch ( slapMode & SLAP_MODE ) {
 	case SLAP_SERVER_MODE:
-
 	case SLAP_TOOL_MODE:
-
-		ldap_pvt_thread_mutex_destroy( &slap_counters.sc_sent_mutex );
-		ldap_pvt_thread_mutex_destroy( &slap_counters.sc_ops_mutex );
-		ldap_pvt_mp_clear( slap_counters.sc_bytes );
-		ldap_pvt_mp_clear( slap_counters.sc_pdu );
-		ldap_pvt_mp_clear( slap_counters.sc_entries );
-		ldap_pvt_mp_clear( slap_counters.sc_refs );
-		ldap_pvt_mp_clear( slap_counters.sc_ops_initiated );
-		ldap_pvt_mp_clear( slap_counters.sc_ops_completed );
-
-#ifdef SLAPD_MONITOR
-		for ( i = 0; i < SLAP_OP_LAST; i++ ) {
-			ldap_pvt_mp_clear( slap_counters.sc_ops_initiated_[ i ] );
-			ldap_pvt_mp_clear( slap_counters.sc_ops_completed_[ i ] );
-		}
-#endif /* SLAPD_MONITOR */
+		slap_counters_destroy( &slap_counters );
 		break;
 
 	default:
 		Debug( LDAP_DEBUG_ANY,
-			"slap_destroy(): undefined mode (%d).\n", slapMode, 0, 0 );
+			"slap_destroy(): undefined mode (%d).\n", slapMode );
 
 		rc = 1;
 		break;
@@ -316,3 +283,42 @@ int slap_destroy(void)
 	/* should destroy the above mutex */
 	return rc;
 }
+
+void slap_counters_init( slap_counters_t *sc )
+{
+	int i;
+
+	ldap_pvt_thread_mutex_init( &sc->sc_mutex );
+	ldap_pvt_mp_init( sc->sc_bytes );
+	ldap_pvt_mp_init( sc->sc_pdu );
+	ldap_pvt_mp_init( sc->sc_entries );
+	ldap_pvt_mp_init( sc->sc_refs );
+
+	ldap_pvt_mp_init( sc->sc_ops_initiated );
+	ldap_pvt_mp_init( sc->sc_ops_completed );
+
+	for ( i = 0; i < SLAP_OP_LAST; i++ ) {
+		ldap_pvt_mp_init( sc->sc_ops_initiated_[ i ] );
+		ldap_pvt_mp_init( sc->sc_ops_completed_[ i ] );
+	}
+}
+
+void slap_counters_destroy( slap_counters_t *sc )
+{
+	int i;
+
+	ldap_pvt_thread_mutex_destroy( &sc->sc_mutex );
+	ldap_pvt_mp_clear( sc->sc_bytes );
+	ldap_pvt_mp_clear( sc->sc_pdu );
+	ldap_pvt_mp_clear( sc->sc_entries );
+	ldap_pvt_mp_clear( sc->sc_refs );
+
+	ldap_pvt_mp_clear( sc->sc_ops_initiated );
+	ldap_pvt_mp_clear( sc->sc_ops_completed );
+
+	for ( i = 0; i < SLAP_OP_LAST; i++ ) {
+		ldap_pvt_mp_clear( sc->sc_ops_initiated_[ i ] );
+		ldap_pvt_mp_clear( sc->sc_ops_completed_[ i ] );
+	}
+}
+
