@@ -382,8 +382,8 @@ dynlist_nested_memberOf( Entry *e, AttributeDescription *ad, TAvlnode *sups )
 	Attribute *a;
 
 	a = attr_find( e->e_attrs, ad );
-	for ( ptr = tavl_end( sups, TAVL_DIR_LEFT ); ptr;
-		ptr = tavl_next( ptr, TAVL_DIR_RIGHT )) {
+	for ( ptr = ldap_tavl_end( sups, TAVL_DIR_LEFT ); ptr;
+		ptr = ldap_tavl_next( ptr, TAVL_DIR_RIGHT )) {
 		dyn = ptr->avl_data;
 		if ( a ) {
 			unsigned slot;
@@ -447,10 +447,10 @@ dynlist_nested_member( Operation *op, dynlist_member_t *dm, TAvlnode *subs )
 	if ( !a )
 		return;
 
-	for ( ptr = tavl_end( subs, TAVL_DIR_LEFT ); ptr;
-		ptr = tavl_next( ptr, TAVL_DIR_RIGHT )) {
+	for ( ptr = ldap_tavl_end( subs, TAVL_DIR_LEFT ); ptr;
+		ptr = ldap_tavl_next( ptr, TAVL_DIR_RIGHT )) {
 		dyn = ptr->avl_data;
-		if ( tavl_insert( &dm->dm_groups, dyn, dynlist_ptr_cmp, avl_dup_error ))
+		if ( ldap_tavl_insert( &dm->dm_groups, dyn, dynlist_ptr_cmp, ldap_avl_dup_error ))
 			continue;
 		if ( overlay_entry_get_ov( op, &dyn->dy_name, NULL, NULL, 0, &ne, on ) != LDAP_SUCCESS || ne == NULL )
 			continue;
@@ -764,7 +764,7 @@ checkdyn:
 					dm.dm_ad = dlm->dlm_member_ad;
 					dynlist_nested_member( op, &dm, dyn->dy_subs );
 					if ( dm.dm_groups )
-						tavl_free( dm.dm_groups, NULL );
+						ldap_tavl_free( dm.dm_groups, NULL );
 				}
 			}
 		}
@@ -1089,7 +1089,7 @@ dynlist_search1resp( Operation *op, SlapReply *rs )
 			if ( b )
 				dyn->dy_staticmember = ds->ds_dlm->dlm_member_ad;
 
-			if ( tavl_insert( &ds->ds_names, dyn, dynlist_avl_cmp, avl_dup_error )) {
+			if ( ldap_tavl_insert( &ds->ds_names, dyn, dynlist_avl_cmp, ldap_avl_dup_error )) {
 				for (i=dyn->dy_numuris-1; i>=0; i--) {
 					ludp = dyn->dy_uris[i];
 					if ( ludp->lud_filter ) {
@@ -1243,7 +1243,7 @@ dynlist_filter_group( Operation *op, dynlist_name_t *dyn, Filter *n, dynlist_sea
 	Attribute *a;
 	int rc = -1;
 
-	if ( tavl_insert( &ds->ds_fnodes, dyn, dynlist_ptr_cmp, avl_dup_error ))
+	if ( ldap_tavl_insert( &ds->ds_fnodes, dyn, dynlist_ptr_cmp, ldap_avl_dup_error ))
 		return 0;
 
 	if ( overlay_entry_get_ov( op, &dyn->dy_name, NULL, NULL, 0, &e, on ) !=
@@ -1264,8 +1264,8 @@ dynlist_filter_group( Operation *op, dynlist_name_t *dyn, Filter *n, dynlist_sea
 	overlay_entry_release_ov( op, e, 0, on );
 	if ( dyn->dy_subs && !rc ) {
 		TAvlnode *ptr;
-		for ( ptr = tavl_end( dyn->dy_subs, TAVL_DIR_LEFT ); ptr;
-			ptr = tavl_next( ptr, TAVL_DIR_RIGHT )) {
+		for ( ptr = ldap_tavl_end( dyn->dy_subs, TAVL_DIR_LEFT ); ptr;
+			ptr = ldap_tavl_next( ptr, TAVL_DIR_RIGHT )) {
 			dyn = ptr->avl_data;
 			rc = dynlist_filter_group( op, dyn, n, ds );
 			if ( rc )
@@ -1300,7 +1300,7 @@ dynlist_filter_dup( Operation *op, Filter *f, AttributeDescription *ad, dynlist_
 	case LDAP_FILTER_EQUALITY:
 		n->f_choice = SLAPD_FILTER_COMPUTED;
 		if ( f->f_av_desc == ad ) {
-			dynlist_name_t *dyn = tavl_find( ds->ds_names, &f->f_av_value, dynlist_avl_cmp );
+			dynlist_name_t *dyn = ldap_tavl_find( ds->ds_names, &f->f_av_value, dynlist_avl_cmp );
 			if ( dyn && !dynlist_filter_group( op, dyn, n, ds ))
 				break;
 		}
@@ -1380,9 +1380,9 @@ dynlist_search_free( void *ptr )
 		ldap_free_urldesc( ludp );
 	}
 	if ( dyn->dy_subs )
-		tavl_free( dyn->dy_subs, NULL );
+		ldap_tavl_free( dyn->dy_subs, NULL );
 	if ( dyn->dy_sups )
-		tavl_free( dyn->dy_sups, NULL );
+		ldap_tavl_free( dyn->dy_sups, NULL );
 	ch_free( ptr );
 }
 
@@ -1393,9 +1393,9 @@ dynlist_search_cleanup( Operation *op, SlapReply *rs )
 		rs->sr_err == SLAPD_ABANDON ) {
 		slap_callback *sc = op->o_callback;
 		dynlist_search_t *ds = op->o_callback->sc_private;
-		tavl_free( ds->ds_names, dynlist_search_free );
+		ldap_tavl_free( ds->ds_names, dynlist_search_free );
 		if ( ds->ds_fnodes )
-			tavl_free( ds->ds_fnodes, NULL );
+			ldap_tavl_free( ds->ds_fnodes, NULL );
 		if ( ds->ds_origfilter ) {
 			op->o_tmpfree( op->ors_filterstr.bv_val, op->o_tmpmemctx );
 			dynlist_filter_free( op, op->ors_filter );
@@ -1472,8 +1472,8 @@ dynlist_add_memberOf(Operation *op, SlapReply *rs, dynlist_search_t *ds)
 	Attribute *a;
 
 	/* See if there are any memberOf values to attach to this entry */
-	for ( ptr = tavl_end( ds->ds_names, TAVL_DIR_LEFT ); ptr;
-		ptr = tavl_next( ptr, TAVL_DIR_RIGHT )) {
+	for ( ptr = ldap_tavl_end( ds->ds_names, TAVL_DIR_LEFT ); ptr;
+		ptr = ldap_tavl_next( ptr, TAVL_DIR_RIGHT )) {
 		dynlist_map_t *dlm;
 		dyn = ptr->avl_data;
 		for ( dlm = dyn->dy_dli->dli_dlm; dlm; dlm = dlm->dlm_next ) {
@@ -1520,7 +1520,7 @@ dynlist_search2resp( Operation *op, SlapReply *rs )
 	if ( rs->sr_type == REP_SEARCH && rs->sr_entry != NULL ) {
 		rc = SLAP_CB_CONTINUE;
 		/* See if this is one of our dynamic entries */
-		dyn = tavl_find( ds->ds_names, &rs->sr_entry->e_nname, dynlist_avl_cmp );
+		dyn = ldap_tavl_find( ds->ds_names, &rs->sr_entry->e_nname, dynlist_avl_cmp );
 		if ( dyn ) {
 			dyn->dy_seen = 1;
 			rc = dynlist_prepare_entry( op, rs, dyn->dy_dli, dyn );
@@ -1542,8 +1542,8 @@ dynlist_search2resp( Operation *op, SlapReply *rs )
 		/* Check for any unexpanded dynamic group entries that weren't picked up
 		 * by the original search filter.
 		 */
-		for ( ptr = tavl_end( ds->ds_names, TAVL_DIR_LEFT ); ptr;
-			ptr = tavl_next( ptr, TAVL_DIR_RIGHT )) {
+		for ( ptr = ldap_tavl_end( ds->ds_names, TAVL_DIR_LEFT ); ptr;
+			ptr = ldap_tavl_next( ptr, TAVL_DIR_RIGHT )) {
 			dyn = ptr->avl_data;
 			if ( dyn->dy_seen )
 				continue;
@@ -1599,13 +1599,13 @@ dynlist_nestlink_dg( Operation *op, SlapReply *rs )
 	if ( rs->sr_type != REP_SEARCH )
 		return LDAP_SUCCESS;
 
-	dj = tavl_find( dll->dl_ds->ds_names, &rs->sr_entry->e_nname, dynlist_avl_cmp );
+	dj = ldap_tavl_find( dll->dl_ds->ds_names, &rs->sr_entry->e_nname, dynlist_avl_cmp );
 	if ( dj ) {
 		if ( ds->ds_want & WANT_MEMBEROF ) {
-			tavl_insert( &dj->dy_sups, di, dynlist_ptr_cmp, avl_dup_error );
+			ldap_tavl_insert( &dj->dy_sups, di, dynlist_ptr_cmp, ldap_avl_dup_error );
 		}
 		if ( ds->ds_want & WANT_MEMBER ) {
-			tavl_insert( &di->dy_subs, dj, dynlist_ptr_cmp, avl_dup_error );
+			ldap_tavl_insert( &di->dy_subs, dj, dynlist_ptr_cmp, ldap_avl_dup_error );
 		}
 	}
 	return LDAP_SUCCESS;
@@ -1622,8 +1622,8 @@ dynlist_nestlink( Operation *op, dynlist_search_t *ds )
 	Attribute *a;
 	int i;
 
-	for ( ptr = tavl_end( ds->ds_names, TAVL_DIR_LEFT ); ptr;
-		ptr = tavl_next( ptr, TAVL_DIR_RIGHT )) {
+	for ( ptr = ldap_tavl_end( ds->ds_names, TAVL_DIR_LEFT ); ptr;
+		ptr = ldap_tavl_next( ptr, TAVL_DIR_RIGHT )) {
 		di = ptr->avl_data;
 		if ( ds->ds_dlm ) {
 			if ( overlay_entry_get_ov( op, &di->dy_name, NULL, NULL, 0, &e, on ) != LDAP_SUCCESS || e == NULL )
@@ -1631,13 +1631,13 @@ dynlist_nestlink( Operation *op, dynlist_search_t *ds )
 			a = attr_find( e->e_attrs, ds->ds_dlm->dlm_member_ad );
 			if ( a ) {
 				for ( i=0; i < a->a_numvals; i++ ) {
-					dj = tavl_find( ds->ds_names, &a->a_nvals[i], dynlist_avl_cmp );
+					dj = ldap_tavl_find( ds->ds_names, &a->a_nvals[i], dynlist_avl_cmp );
 					if ( dj ) {
 						if ( ds->ds_want & WANT_MEMBEROF ) {
-							tavl_insert( &dj->dy_sups, di, dynlist_ptr_cmp, avl_dup_error );
+							ldap_tavl_insert( &dj->dy_sups, di, dynlist_ptr_cmp, ldap_avl_dup_error );
 						}
 						if ( ds->ds_want & WANT_MEMBER ) {
-							tavl_insert( &di->dy_subs, dj, dynlist_ptr_cmp, avl_dup_error );
+							ldap_tavl_insert( &di->dy_subs, dj, dynlist_ptr_cmp, ldap_avl_dup_error );
 						}
 					}
 				}
